@@ -138,7 +138,7 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
    */
   RenamePropertiesWithModuleSupport(AbstractCompiler compiler, boolean generatePseudoNames,
       NameGenerator nameGenerator) {
-    this(compiler, generatePseudoNames, null, null, null, nameGenerator);
+    this(compiler, generatePseudoNames, null, null, null, nameGenerator, null);
   }
 
   /**
@@ -156,9 +156,11 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
   RenamePropertiesWithModuleSupport(AbstractCompiler compiler,
       boolean generatePseudoNames, VariableMap prevUsedPropertyMap,
       NameGenerator nameGenerator) {
-    this(compiler, generatePseudoNames, prevUsedPropertyMap, null, null, nameGenerator);
+    this(compiler, generatePseudoNames, prevUsedPropertyMap, null, null, nameGenerator, null);
   }
 
+  private Set<String> propertyNamesToKeep;
+
   /**
    * Creates an instance.
    *
@@ -179,13 +181,15 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
       VariableMap prevUsedPropertyMap,
       @Nullable char[] reservedFirstCharacters,
       @Nullable char[] reservedNonFirstCharacters,
-      NameGenerator nameGenerator) {
+      NameGenerator nameGenerator,
+      Set<String> propertyNamesToKeep) {
     this.compiler = compiler;
     this.generatePseudoNames = generatePseudoNames;
     this.prevUsedPropertyMap = prevUsedPropertyMap;
     this.reservedFirstCharacters = reservedFirstCharacters;
     this.reservedNonFirstCharacters = reservedNonFirstCharacters;
     this.nameGenerator = nameGenerator;
+    this.propertyNamesToKeep = propertyNamesToKeep;
     externedNames.addAll(compiler.getExternProperties());
   }
 
@@ -226,6 +230,9 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
      */
     reservedNames.addAll(externedNames);
     reservedNames.addAll(quotedNames);
+    if (propertyNamesToKeep != null) {
+      reservedNames.addAll(propertyNamesToKeep);
+    }
     
     generateNames(propsByFreq, reservedNames);
 
@@ -364,6 +371,8 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
                 propNode.getString())) {
               externedNames.add(propNode.getString());
               break;
+            } else if(propertyNamesToKeep != null && propertyNamesToKeep.contains(propNode.getString())) {
+              break;
             }
             maybeMarkCandidate(propNode);
           }
@@ -378,7 +387,9 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
               // that could conflict with this quoted key.
               quotedNames.add(key.getString());
             } else if (compiler.getCodingConvention().blockRenamingForProperty(key.getString())) {
-              externedNames.add(key.getString());
+              externedNames.add(key.getString()); 
+            } else if(propertyNamesToKeep != null && propertyNamesToKeep.contains(key.getString())) {
+                continue;
             } else {
               maybeMarkCandidate(key);
             }
@@ -396,6 +407,8 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
               quotedNames.add(key.getString());
             } else if (compiler.getCodingConvention().blockRenamingForProperty(key.getString())) {
               externedNames.add(key.getString());
+            } else if(propertyNamesToKeep != null && propertyNamesToKeep.contains(key.getString())) {
+                continue;
             } else {
               maybeMarkCandidate(key);
             }
@@ -437,6 +450,8 @@ class RenamePropertiesWithModuleSupport implements CompilerPass {
                   Node fnName = member.getFirstChild();
                   if (compiler.getCodingConvention().blockRenamingForProperty(memberDefName)) {
                     externedNames.add(fnName.getString());
+                  } else if(propertyNamesToKeep != null && propertyNamesToKeep.contains(memberDefName)) {
+                      continue;
                   } else if (memberDefName.equals("constructor")
                       || memberDefName.equals("superClass_")) {
                     // TODO (simarora) is there a better way to identify these externs?
